@@ -21,8 +21,6 @@
  */
 
 
-#define LOWERCASE
-
 #include "sysincludes.h"
 #include "msdos.h"
 #include "mtools.h"
@@ -78,16 +76,15 @@ typedef struct Arg_t {
 	int noClobber;
 } Arg_t;
 
-static int _unix_write(direntry_t *entry, MainParam_t *mp, int needfilter,
-		       const char *unixFile);
+static int _unix_write(MainParam_t *mp, int needfilter, const char *unixFile);
 
 /* Write the Unix file */
-static int unix_write(direntry_t *entry, MainParam_t *mp, int needfilter)
+static int unix_write(MainParam_t *mp, int needfilter)
 {
 	Arg_t *arg=(Arg_t *) mp->arg;
 
 	if(arg->type)
-		return _unix_write(entry, mp, needfilter, "-");
+		return _unix_write(mp, needfilter, "-");
 	else {
 		char *unixFile = mpBuildUnixFilename(mp);
 		int ret;
@@ -95,7 +92,7 @@ static int unix_write(direntry_t *entry, MainParam_t *mp, int needfilter)
 			printOom();
 			return ERROR_ONE;
 		}
-		ret = _unix_write(entry, mp, needfilter, unixFile);
+		ret = _unix_write(mp, needfilter, unixFile);
 		free(unixFile);
 		return ret;
 	}
@@ -103,15 +100,14 @@ static int unix_write(direntry_t *entry, MainParam_t *mp, int needfilter)
 
 
 /* Write the Unix file */
-static int _unix_write(direntry_t *entry, MainParam_t *mp, int needfilter,
-		       const char *unixFile)
+static int _unix_write(MainParam_t *mp, int needfilter, const char *unixFile)
 {
 	Arg_t *arg=(Arg_t *) mp->arg;
 	time_t mtime;
 	Stream_t *File=mp->File;
 	Stream_t *Target, *Source;
 	struct MT_STAT stbuf;
-	int ret;
+	ssize_t ret;
 	char errmsg[80];
 
 	File->Class->get_data(File, &mtime, 0, 0, 0);
@@ -121,7 +117,7 @@ static int _unix_write(direntry_t *entry, MainParam_t *mp, int needfilter,
 
 	/* if we are creating a file, check whether it already exists */
 	if(!arg->type) {
-		if (!arg->nowarn && &arg->type && !access(unixFile, 0)){
+		if (!arg->nowarn && !access(unixFile, 0)){
 			if(arg->noClobber) {
 				fprintf(stderr, "File \"%s\" exists. To overwrite, try again, and explicitly specify target directory\n",unixFile);
 				return ERROR_ONE;
@@ -265,15 +261,15 @@ static int unix_copydir(direntry_t *entry, MainParam_t *mp)
 	}
 }
 
-static  int dos_to_unix(direntry_t *entry, MainParam_t *mp)
+static  int dos_to_unix(direntry_t *entry UNUSEDP, MainParam_t *mp)
 {
-	return unix_write(entry, mp, 1);
+	return unix_write(mp, 1);
 }
 
 
 static  int unix_to_unix(MainParam_t *mp)
 {
-	return unix_write(0, mp, 0);
+	return unix_write(mp, 0);
 }
 
 
@@ -293,7 +289,9 @@ static int writeit(struct dos_name_t *dosname,
 {
 	Stream_t *Target;
 	time_t now;
-	int type, fat, ret;
+	int type;
+	ssize_t ret;
+	uint32_t fat;
 	time_t date;
 	mt_size_t filesize, newsize;
 	Arg_t *arg = (Arg_t *) arg0;
@@ -306,7 +304,7 @@ static int writeit(struct dos_name_t *dosname,
 		return -1;
 	}
 
-	if(fileTooBig(filesize)) {
+	if(fileSizeTooBig(filesize)) {
 		fprintf(stderr, "File \"%s\" too big\n", longname);
 		return 1;
 	}
@@ -355,7 +353,7 @@ static int writeit(struct dos_name_t *dosname,
 		fat_free(arg->mp.targetDir, fat);
 		return -1;
 	} else {
-		mk_entry(dosname, arg->attr, fat, truncBytes32(newsize),
+		mk_entry(dosname, arg->attr, fat, (uint32_t)newsize,
 				 now, &entry->dir);
 		return 0;
 	}
@@ -396,7 +394,7 @@ static Stream_t *subDir(Stream_t *parent, const char *filename)
 	direntry_t entry;
 	initializeDirentry(&entry, parent);
 
-	switch(vfat_lookup(&entry, filename, -1, ACCEPT_DIR, 0, 0)) {
+	switch(vfat_lookup_zt(&entry, filename, ACCEPT_DIR, 0, 0, 0, 0)) {
 	    case 0:
 		return OpenFileByDirentry(&entry);
 	    case -1:
@@ -504,11 +502,11 @@ static void usage(int ret)
 	exit(ret);
 }
 
+void mcopy(int argc, char **argv, int mtype) NORETURN;
 void mcopy(int argc, char **argv, int mtype)
 {
 	Arg_t arg;
-	int c, ret, fastquit;
-	int todir;
+	int c, fastquit;
 	
 
 	/* get command line options */
@@ -516,7 +514,6 @@ void mcopy(int argc, char **argv, int mtype)
 	init_clash_handling(& arg.ch);
 
 	/* get command line options */
-	todir = 0;
 	arg.recursive = 0;
 	arg.preserveTime = 0;
 	arg.preserveAttributes = 0;
@@ -531,7 +528,7 @@ void mcopy(int argc, char **argv, int mtype)
 	while ((c = getopt(argc, argv, "i:abB/sptTnmvQD:oh")) != EOF) {
 		switch (c) {
 			case 'i':
-				set_cmd_line_image(optarg, 0);
+				set_cmd_line_image(optarg);
 				break;
 			case 's':
 			case '/':
@@ -542,6 +539,7 @@ void mcopy(int argc, char **argv, int mtype)
 				break;
 			case 'T':
 				arg.convertCharset = 1;
+				 /*-fallthrough*/
 			case 'a':
 			case 't':
 				arg.textmode = 1;
@@ -563,7 +561,7 @@ void mcopy(int argc, char **argv, int mtype)
 				batchmode = 1;
 				break;
 			case 'o':
-				handle_clash_options(&arg.ch, c);
+				handle_clash_options(&arg.ch, (char) c);
 				break;
 			case 'D':
 				if(handle_clash_options(&arg.ch, *optarg))
@@ -575,6 +573,7 @@ void mcopy(int argc, char **argv, int mtype)
 				usage(1);
 			default:
 				break;
+
 		}
 	}
 
@@ -613,7 +612,7 @@ void mcopy(int argc, char **argv, int mtype)
 			target = argv[argc];
 		}
 
-		ret = target_lookup(&arg.mp, target);
+		target_lookup(&arg.mp, target);
 		if(!arg.mp.targetDir && !arg.mp.unixTarget) {
 			fprintf(stderr,"Bad target %s\n", target);
 			exit(1);
