@@ -23,19 +23,35 @@ static Dword read_dword(int handle)
 	Byte val[4];
 	
 	if(read(handle, (char *)val, 4) < 4)
-		return -1;
+		return (Dword) -1;
 
 	return byte2dword(val);
 }
 
-UNUSED(static Qword read_qword(int handle) )
+UNUSED(static int32_t read_sdword(int handle))
+{
+	Byte val[4];
+	
+	if(read(handle, (char *)val, 4) < 4)
+		return (int32_t) -1;
+
+	return byte2sdword(val);
+}
+
+
+struct SQwordRet { int64_t v; int err; };
+UNUSED(static struct SQwordRet read_sqword(int handle) )
 {
 	Byte val[8];
+	struct SQwordRet ret;
 	
-	if(read(handle, (char *)val, 8) < 8)
-		return -1;
-
-	return byte2qword(val);
+	if(read(handle, (char *)val, 8) < 8) {
+		ret.err=-1;
+	} else {
+		ret.v = (int64_t) byte2qword(val);
+		ret.err = 0;
+	}
+	return ret;
 }
 
 #endif
